using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using Dart.LiveControls;

namespace LiveSamples
{
	/// <summary>
	/// Summary description for LiveAuction.
	/// </summary>
	public class LiveAuction : System.Web.UI.Page
	{
		protected Dart.LiveControls.LiveLabel lblTitle;
		protected Dart.LiveControls.LiveLabel lblAbout;
		protected Dart.LiveControls.LiveButton cmdStart;
		protected Dart.LiveControls.LivePanel panelAbout;
		protected Dart.LiveControls.LiveImageButton LiveImageButton1;
		protected Dart.LiveControls.LiveLabel lblCurrentBid;
		protected Dart.LiveControls.LiveLabel lblTimeLeft;
		protected Dart.LiveControls.LiveLabel lblStartTime;
		protected Dart.LiveControls.LiveLabel lblHistory;
		protected Dart.LiveControls.LiveLabel lblHighBidder;
		protected Dart.LiveControls.LivePanel panelAuctionInfo;
		protected Dart.LiveControls.LiveMessageBox LiveMessageBox1;
		protected Dart.LiveControls.LiveTimer LiveTimer1;
	
		private void Page_Load(object sender, System.EventArgs e)
		{
			if(!IsPostBack)
			{
				Session["StartTime"] = null;
				Session["EndTime"] = null;
				string s = "This sample simulates the last 2 minutes of an online auction. Click 'Start Simulation' to start. <p>";
				s += "Once started, this page will change into a page resembling that of a well-known auction site. <p>";
				s += "The difference between this page and others is that the information is updated in real time using a LiveTimer control.";
				lblAbout.Text = s;
				lblTitle.Text = "LiveAuction Sample";
				panelAuctionInfo.Visible = false;
				lblHighBidder.ForeColor = Color.Black;
			}
		}

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
            if (!this.DesignMode)
                InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.cmdStart.Click += new System.EventHandler(this.cmdStart_Click);
			this.LiveImageButton1.Click += new System.Web.UI.ImageClickEventHandler(this.LiveImageButton1_Click);
			this.LiveTimer1.Tick += new System.EventHandler(this.LiveTimer1_Tick);
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion

		private void cmdStart_Click(object sender, System.EventArgs e)
		{
			// Hide the about panel
			panelAbout.Visible = false;
			// Show the auction panel
			panelAuctionInfo.Visible = true;
			// Set simulated end time to 2 minutes from now
			Session["EndTime"] = System.DateTime.Now.Add(new TimeSpan(0, 2, 0));
			// Calculate simulated start time (5 days before end)
			Session["StartTime"] = ((DateTime)Session["EndTime"]).Subtract(new TimeSpan(5, 0, 0, 0));
			// Set the labels so that it looks like the auction is almost over
			lblTitle.Text = "1952 Original Fender Telecaster";
			lblStartTime.Text = ((DateTime)Session["StartTime"]).ToLongDateString() + " " + ((DateTime)Session["StartTime"]).ToLongTimeString();
			lblCurrentBid.Text = "US $4,320.00";
			lblHistory.Text = "27 bids";
			lblHighBidder.Text = GetHighBidder();
			lblTimeLeft.Text = "00:02:00";
			// Start the LiveTimer
			LiveTimer1.Blocking = true;
			LiveTimer1.Interval = 500;
			LiveTimer1.Start();
		}

		private string GetHighBidder()
		{
			switch(LiveShared.GetRandomNumber(0, 5))
			{
				case 0:
					return "livectrl_guru";
				case 1:
					return "fenderman";
				case 2:
					return "teleluvr";
				case 3:
					return "coder229";
				case 4:
					return "freddy342";
				case 5:
					return "soccermom62";
			}
			return "";
		}

		private void EndAuction()
		{
			// Set the labels for the end of the auction
			lblTimeLeft.Text = "Ended";
			lblHighBidder.ForeColor = Color.DarkRed;
		}

		private void RecordRandomBid()
		{
			// Pick a random number between 0 and 6. If it's not 0 there was no bid
			if (LiveShared.GetRandomNumber(0, 6) != 0)
				return;
			// Determine the current bid (between 1% and 5% of the current)
			double RaiseFactor = 1 + (LiveShared.GetRandomNumber(1, 5) / (double)100);
			int TotalBids = 0;
			try
			{
				TotalBids = Convert.ToInt32(lblHistory.Text.Replace(" bids", "")) + 1;
			}
			catch{}
			double CurrentBid = 0;
			try
			{
				string s = lblCurrentBid.Text.Replace("US $", "").Replace(",", "");
				CurrentBid = (double)Convert.ToDecimal(s);
			}
			catch{}
			CurrentBid *= RaiseFactor;
			lblCurrentBid.Text = "US " + CurrentBid.ToString("C");
			// Set the number of bids
			lblHistory.Text = TotalBids.ToString() + " bids";
			// Set the high bidder
			lblHighBidder.Text = GetHighBidder();
		}

		private void LiveImageButton1_Click(object sender, System.Web.UI.ImageClickEventArgs e)
		{
			// Show a LiveMessageBox when the user clicks the LiveImageButton
			LiveMessageBox1.Show("You just clicked a LiveImageButton! Had this been a real site instead of some cheesy demo, surely something really awesome would have happened!", DialogType.Alert);
    	}

		private void LiveTimer1_Tick(object sender, System.EventArgs e)
		{
			// Stop the timer. The timer will only be restarted if the Auction is not over
			LiveTimer1.Stop();
			// Determine how much time is remaining
			TimeSpan TimeLeft = new TimeSpan(((DateTime)Session["EndTime"]).Ticks - DateTime.Now.Ticks);
			if(TimeLeft.Ticks > 0)
			{
				// Set the time left LiveLabel
				lblTimeLeft.Text = TimeLeft.Hours.ToString("D2") + ":" + TimeLeft.Minutes.ToString("D2") + ":" + TimeLeft.Seconds.ToString("D2");
				// Record a simulated bid
				RecordRandomBid();
				// Restart the Timer
				LiveTimer1.Start();
			}
			else
			{
				// The auction is over now
				EndAuction();
			}
		}
	}
}
