using System;
using System.IO;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;

namespace LiveSamples
{
	/// <summary>
	/// Summary description for LiveSlideShow.
	/// </summary>
	public class LiveSlideShow : System.Web.UI.Page
	{
		private string baseURL;
		protected Dart.LiveControls.LiveImage liNowShowing;
		protected Dart.LiveControls.LivePanel LivePanel1;
		protected Dart.LiveControls.LiveDropDownList lddlSlides;
		protected Dart.LiveControls.LiveImageButton lbPlay;
		protected Dart.LiveControls.LiveImageButton lbPause;
		protected Dart.LiveControls.LiveImageButton lbPrevious;
		protected Dart.LiveControls.LiveImageButton lbNext;
		protected Dart.LiveControls.LiveDataGrid ldgSlides;
		protected System.Web.UI.WebControls.Image imgPlay;
		protected System.Web.UI.WebControls.Image imgPause;
		protected System.Web.UI.WebControls.Image imgPrev;
		protected System.Web.UI.WebControls.Image imgNext;
		protected Dart.LiveControls.LiveTimer LiveTimer1;
	
		private void Page_Load(object sender, System.EventArgs e)
		{
			baseURL = Request.ApplicationPath + "/";
			if (baseURL.StartsWith("//"))
                baseURL = baseURL.Replace("//", "/");
			if (!Page.IsPostBack)
			{
				lbPlay.AccessKey = "N";
				lbPlay.ToolTip = "Play";
				lbPlay.ImageUrl = getUrlString("images/play.jpg");
				imgPlay.ImageUrl = lbPlay.ImageUrl;

				lbPause.AccessKey = "M";
				lbPause.ToolTip = "Pause";
				lbPause.ImageUrl = getUrlString("images/pause.jpg");
				imgPause.ImageUrl = lbPause.ImageUrl;
				
				lbPrevious.AccessKey = "<";
				lbPrevious.ToolTip = "Previous";
				lbPrevious.ImageUrl = getUrlString("images/previous.jpg");
				imgPrev.ImageUrl = lbPrevious.ImageUrl;

				lbNext.AccessKey = ">";
				lbNext.ToolTip = "Next";
				lbNext.ImageUrl = getUrlString("images/next.jpg");
				imgNext.ImageUrl = lbNext.ImageUrl;
				
				//build initial list of images
				RefreshImageList();
				DrawPage();
				InitTimer();
			}
		}


		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
            if (!this.DesignMode) 
                InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			liNowShowing.DoubleClick += new Dart.LiveControls.LocationEventHandler(liNowShowing_DoubleClick);
			lddlSlides.SelectedIndexChanged += new System.EventHandler(lddlSlides_SelectedIndexChanged);
			lbPlay.Click += new System.Web.UI.ImageClickEventHandler(lbPlay_Click);
			lbPause.Click += new System.Web.UI.ImageClickEventHandler(lbPause_Click);
			lbPrevious.Click += new System.Web.UI.ImageClickEventHandler(lbPrevious_Click);
			lbNext.Click += new System.Web.UI.ImageClickEventHandler(lbNext_Click);
			ldgSlides.ItemCommand += new System.Web.UI.WebControls.DataGridCommandEventHandler(ldgSlides_ItemCommand);
			ldgSlides.PageIndexChanged += new System.Web.UI.WebControls.DataGridPageChangedEventHandler(ldgSlides_PageIndexChanged);
			LiveTimer1.Tick += new System.EventHandler(LiveTimer1_Tick);
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion

		#region Initialization
		private void InitTimer()
		{
			LivePanel1.BackColor = Color.White;
			LivePanel1.Height = liNowShowing.Height;
			TimerCount = 0;
		}
		private void RefreshImageList()
		{	
			string SlidePath = Server.MapPath(getUrlString("slides"));
			DirectoryInfo d = new DirectoryInfo(SlidePath);

			//prep dataset
			if (SlideData.Tables.Count == 0)
			{
				DataTable dt = SlideData.Tables.Add("Images");
				dt.Columns.Add("url");
				dt.Columns.Add("name");
				dt.Columns.Add("size");

				//load data
				foreach(FileInfo f in d.GetFiles())
				{
					if((f.Name.ToUpper().EndsWith("JPG")) || (f.Name.ToUpper().EndsWith("GIF")))
					{
						SlidesArray.Add(f.Name);
						dt.Rows.Add(new object[]{getUrlString("slides/") + f.Name, f.Name, (f.Length/1024) + "KB"});
					}
				}
			}
			SlideIndex = 0;
			PageIndex = 0;
			ShowImage();
			lddlSlides.DataSource = SlidesArray;
			lddlSlides.DataBind();
		}
		#endregion

		#region Event Handlers
		private void lbPlay_Click(object sender, System.Web.UI.ImageClickEventArgs e)
		{
			if(lbPlay.ImageUrl.ToLower() == getUrlString("images/play-selected.jpg"))
				return;
			lbPlay.ImageUrl = getUrlString("images/play-selected.jpg");
			lbPause.ImageUrl = getUrlString("images/pause.jpg");
			DrawTimer();
			LiveTimer1.Start();
		}
		private void lbPause_Click(object sender, System.Web.UI.ImageClickEventArgs e)
		{
			if(lbPause.ImageUrl.ToLower() == getUrlString("images/pause-selected.jpg"))
				return;
			lbPause.ImageUrl = getUrlString("images/pause-selected.jpg");
			lbPlay.ImageUrl = getUrlString("images/play.jpg");
			LiveTimer1.Stop();
		}
		private void LiveTimer1_Tick(object sender, System.EventArgs e)
		{
			TimerCount++;
			if (TimerCount >= 5 )
			{
				TimerCount = 0;
				Shuttle(1);
			}
			DrawTimer();
		}

		private void liNowShowing_DoubleClick(object sender, Dart.LiveControls.LocationEventArgs e)
		{
			Shuttle(1);
		}

		private void lbPrevious_Click(object sender, System.Web.UI.ImageClickEventArgs e)
		{
			Shuttle(-1);
		}

		private void lbNext_Click(object sender, System.Web.UI.ImageClickEventArgs e)
		{
			Shuttle(1);
		}

		private void ldgSlides_ItemCommand(object source, System.Web.UI.WebControls.DataGridCommandEventArgs e)
		{
			SlideIndex = (PageIndex * ldgSlides.PageSize) + e.Item.ItemIndex;
			ShowImage();
		}		
		private void ldgSlides_PageIndexChanged(object source, System.Web.UI.WebControls.DataGridPageChangedEventArgs e)
		{
			
			PageIndex = e.NewPageIndex;
			int lowerLimit = (PageIndex * ldgSlides.PageSize);
			if ((SlideIndex >= lowerLimit) && (SlideIndex < (lowerLimit+ldgSlides.PageSize)))
				ldgSlides.SelectedIndex = GridIndex;
			else
				ldgSlides.SelectedIndex = -1;
			DrawPage();

		}
		private void lddlSlides_SelectedIndexChanged(object sender, System.EventArgs e)
		{
			SlideIndex = lddlSlides.SelectedIndex;
			ShowImage();
		}	
		#endregion

		#region private Methods
		private void Shuttle(int move)
		{
			if(SlidesExist)
			{
				SlideIndex += move;
				if(SlideIndex < 0)
					SlideIndex = SlidesArray.Count - 1;
				if(SlideIndex > (SlidesArray.Count - 1))
					SlideIndex = 0;
				ShowImage();
			}
		}

		private void ShowImage()
		{
			ldgSlides.SelectedIndex = GridIndex;
			string url = getSlideUrl(SlideIndex);
			if (url != null)
			{
				liNowShowing.ImageUrl =  url;
				liNowShowing.Visible = true;
				InitTimer();
				if (LiveTimer1.Enabled)
					DrawTimer();
				SetPage();
				if (lddlSlides.SelectedIndex != SlideIndex)
					lddlSlides.SelectedIndex = SlideIndex;
			}
		}

		private void SetPage()
		{
			int curPageIndex = PageIndex;
			int newPageIndex;
			if (SlideIndex < ldgSlides.PageSize)
			{
				newPageIndex = 0;
			}
			else
			{
				newPageIndex = (SlideIndex / ldgSlides.PageSize);
			}
			if (newPageIndex != curPageIndex)
			{
				PageIndex = newPageIndex;
				DrawPage();
			}
		}
		private void DrawPage()
		{
			ldgSlides.DataSource = SlideData;
			ldgSlides.CurrentPageIndex = PageIndex;
			ldgSlides.DataBind();
		}

		private void DrawTimer()
		{
			LivePanel1.BackColor = Color.Tomato;
			int cnt = TimerCount;
			int size = (int)(liNowShowing.Height.Value - (cnt * (liNowShowing.Height.Value/4)));
			if (size > 0)
				LivePanel1.Height = new Unit(size,UnitType.Pixel);
			else
				LivePanel1.BackColor = Color.White;
		}

		private string getSlideUrl(int index)
		{
			string f = getSlideFile(index);
			if (f == null)
				return f;
			return getUrlString("/slides/") + f;
		}

		private string getSlideFile(int index)
		{
			if ((SlidesArray.Count > 0) && (index < SlidesArray.Count))
			{
				string slide = (string)SlidesArray[index];
				return slide.StartsWith("/")?slide.Remove(0,1):slide;
			}
			else
			{
				return null;
			}
		}

		private string getUrlString(string relUrl)
		{
			if (relUrl.StartsWith("/"))
				relUrl = relUrl.Remove(0,1);
			relUrl = baseURL + relUrl;
			return relUrl;
		}
		#endregion

		#region public Properties
		public int PageIndex
		{
			get
			{
				return (int)Session["LiveControls.SlideShowCS.PageIndex"];
			}
			set
			{
				Session["LiveControls.SlideShowCS.PageIndex"] = value;
			}
		}

		public int SlideIndex
		{
			get
			{
				return (int)Session["LiveControls.SlideShowCS.SlideIndex"];
			}
			set
			{
				Session["LiveControls.SlideShowCS.SlideIndex"] = value;
			}
		}

		public int TimerCount
		{
			get
			{
				return (int)Session["LiveControls.SlideShowCS.TimerCount"];
			}
			set
			{
				Session["LiveControls.SlideShowCS.TimerCount"] = value;
			}
		}

		public ArrayList SlidesArray
		{
			get
			{
				ArrayList al = (ArrayList)Session["LiveControls.SlideShowCS.SlidesArray"];
				if (al == null)
				{
					al = new ArrayList();
					Session["LiveControls.SlideShowCS.SlidesArray"] = al;
				}
				return al;
			}
		}
		public bool SlidesExist
		{
			get
			{
				return (SlidesArray.Count > 0);
			}
		}
		public int GridIndex
		{
			get
			{
				return SlideIndex % ldgSlides.PageSize;
			}
		}
		public DataSet SlideData
		{
			get
			{
				DataSet ds = (DataSet)Session["LiveControls.SlideShowCS.SlidesDS"];
				if (ds == null)
				{
					ds = new DataSet();
					Session["LiveControls.SlideShowCS.SlidesDS"] = ds;
				}
				return ds;
			}
		}
		#endregion
	}
}
