using System;
using System.IO;

namespace LiveSudoku
{
	/// <summary>
	/// Class to hold puzzle data
	/// </summary>
	public class Puzzle
	{
		int[,] _grid;
		int[,] _workingGrid;
		bool _solved;
		public Puzzle(string path)
		{
			byte[] bytes;
			int r = 0;
			int c = 0;
			int p = 0;

			FileStream file = new FileStream(path, FileMode.Open, FileAccess.Read);
			bytes = new byte[file.Length-1];
			file.Read(bytes,0,bytes.Length);
			file.Close();
	
			while (r < 9)
			{
				c = 0;
				while (c < 9 && p < bytes.Length)
				{
					Grid[r,c] = 0;
					int n = (int)bytes[p++];
					if (n > 48)
					{
						Grid[r,c] = n-48;
						WorkingGrid[r,c] = n-48;
						c++;
					}
					else if (n == 48)
					{
						Grid[r,c] = 0;
						WorkingGrid[r,c] = 0;
						c++;
					}
				}
				r++;
			}
			_solved = false;
		}
		public int[,] Grid
		{
			get
			{
				if (_grid == null)
					_grid = new int[9,9];
				return _grid;
			}
		}
		public int[,] WorkingGrid
		{
			get
			{
				if (_workingGrid == null)
					_workingGrid = new int[9,9];
				return _workingGrid;
			}
		}
		public bool IsSolved
		{
			get
			{
				_solved =  ColsCheck() && RowsCheck() && SubGridsCheck();
				return _solved;
			}
		}

		private bool ColsCheck()
		{
			for (int c = 0; c < 9; c++)
				if (!ColCheck(c))
					return false;
			return true;
		}

		private bool ColCheck(int col)
		{
			int[] Values = new int[9];
			for (int r = 0; r < 9; r++)
			{
				int n = _grid[r,col];
				if (n == 0)
					n = _workingGrid[r,col];
				Values[r] = n;
			}
			return UniqueValueCheck(Values);
		}


		private bool SubGridsCheck()
		{
			for (int r = 0; r < 9; r+=3)
				for (int c = 0; c < 9; c+=3)
					if (!SubGridCheck(r,c))
						return false;
			return true;
		}

		private bool SubGridCheck(int row, int col)
		{
			int[] Values = new int[9];
			int n = 0;
			for (int r = row; r < row + 3; r++)
				for (int c = col; c < col + 3; c++)
				{
					int i = _grid[r,c];
					if (i == 0)
						i = _workingGrid[r,c];
					Values[n++] = i;
				}
			return UniqueValueCheck(Values);
		}

		private bool UniqueValueCheck(int[] Values)
		{
			for (int n1 = 0; n1 < Values.Length; n1++)
				for (int n2 = n1 + 1; n2 < Values.Length; n2++)
				{
					if (Values[n1] == 0)
						return false;
					if (Values[n1] == Values[n2])
						return false;
				}
			return true;
		}

		private bool RowsCheck()
		{
			for (int r = 0; r < 9; r++)
				if (!RowCheck(r))
					return false;
			return true;
		}

		private bool RowCheck(int row)
		{
			int[] Values = new int[9];
			for (int c = 0; c < 9; c++)
			{
				int n = _grid[row,c];
				if (n == 0)
					n = _workingGrid[row,c];
				Values[c] = n;
			}
			return UniqueValueCheck(Values);
		}
	}
}
