using System;
using System.Xml;
using System.Drawing;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using Dart.LiveControls;

namespace LiveSudoku
{
	/// <summary>
	/// Class to hold Skin data to control a puzzle's GUI
	/// </summary>
	public class Skin
	{
		XmlDocument doc;
		string _id, _name, _description;
		public Skin(XmlDocument xd):this(xd.OuterXml)
		{
		}
		public Skin(string skinData)
		{
			try
			{
				doc = new XmlDocument();
				doc.LoadXml(skinData);
				loadData();
			}
			catch
			{
				throw new ArgumentException("Invalid skin file.");
			}
		}
		private void loadData()
		{
			XmlElement docEl = doc.DocumentElement;
			_id = docEl.GetAttribute("id");
			_name = docEl.GetAttribute("name");
			_description = docEl.GetAttribute("description");
		}

		public string ID
		{
			get
			{
				return _id;
			}
		}
		public string Name
		{
			get
			{
				return _name;
			}
		}
		public string Description
		{
			get
			{
				return _description;
			}
		}

		public Color BackColor
		{
			get
			{
				string hex = backGround.GetAttribute("color");
				return StringToColor(hex);
			}
		}

		private Color StringToColor(string hex)
		{
			if (hex != "")
			{
				hex = hex.Replace("#","");
				string hexr = hex.Substring(0,2);
				string hexg = hex.Substring(2,2);
				string hexb = hex.Substring(4,2);
				return Color.FromArgb(Convert.ToInt32(hexr,16),Convert.ToInt32(hexg,16),Convert.ToInt32(hexb,16));
			}
			else
			{
				return Color.Transparent;
			}
		}

		public string BackImageUrl
		{
			get
			{
				return backGround.GetAttribute("imageUrl");
			}
		}

		public string TileExtension
		{
			get
			{
				return tiles.GetAttribute("ext");
			}
		}
		
		public Unit SolvedWidth
		{
			get
			{
				return  new Unit(solved.GetAttribute("width"));
			}
		}
		public Unit SolvedHeight
		{
			get
			{
				return  new Unit(solved.GetAttribute("height"));
			}
		}
		public Unit TileWidth
		{
			get
			{
				return  new Unit(tiles.GetAttribute("width"));
			}
		}
		public Unit TileHeight
		{
			get
			{
				return new Unit(tiles.GetAttribute("height"));
			}
		}
		public Unit MenuWidth
		{
			get
			{
				return new Unit(menu.GetAttribute("width"));
			}
		}
		public int MenuSpacing
		{
			get
			{
				return int.Parse(menu.GetAttribute("spacing"));
			}
		}
		public Unit PopupButtonWidth
		{
			get
			{
				return new Unit(popup.GetAttribute("buttonWidth"));
			}
		}
		public Unit PopupButtonHeight
		{
			get
			{
				return new Unit(popup.GetAttribute("buttonHeight"));
			}
		}
		public Color PopupBackColor
		{
			get
			{
				return StringToColor(popup.GetAttribute("bgcolor"));
			}
		}
		public string MenuLocation
		{
			get
			{
				return menu.GetAttribute("location");
			}
		}

		public Color MenuBackColor
		{
			get
			{
				return StringToColor(menu.GetAttribute("bgcolor"));
			}
		}
		public string MenuBackImageUrl
		{
			get
			{
				return menu.GetAttribute("bgimageUrl");
			}
		}
		
		public void SetControlStyle (WebControl ctl, string style)
		{
			XmlElement el = (XmlElement)menu.SelectSingleNode(style);
			if (el != null)
			{
				ctl.ControlStyle.Reset();
				ctl.MergeStyle(GenControl(el).ControlStyle);
			}
		}
		private XmlElement backGround
		{
			get
			{
				return (XmlElement)doc.SelectSingleNode("skin/background");
			}
		}
		private XmlElement tiles
		{
			get
			{
				return (XmlElement)doc.SelectSingleNode("skin/tiles");
			}
		}
		private XmlElement menu
		{
			get
			{
				return (XmlElement)doc.SelectSingleNode("skin/menu");
			}
		}
		private XmlElement popup
		{
			get
			{
				return (XmlElement)doc.SelectSingleNode("skin/popup");
			}
		}
		private XmlElement solved
		{
			get
			{
				return (XmlElement)doc.SelectSingleNode("skin/solved");
			}
		}
		private WebControl GenControl(XmlElement fontEl)
		{
			LiveLabel l = new LiveLabel();
			l.Font.Name = fontEl.GetAttribute("fontName");
			l.Font.Size = new FontUnit(fontEl.GetAttribute("fontSize"));
			l.Font.Bold = bool.Parse(fontEl.GetAttribute("fontBold"));
			l.ForeColor = StringToColor(fontEl.GetAttribute("fontColor"));
			l.BackColor = StringToColor(fontEl.GetAttribute("bgcolor"));
			return (WebControl)l;
		}
	}
}
