Imports System.IO

Namespace LiveSudoku
	''' <summary>
	''' Class to hold puzzle data
	''' </summary>
	Public Class Puzzle
		Private _grid(,) As Integer
		Private _workingGrid(,) As Integer
		Private _solved As Boolean
		Public Sub New(ByVal path As String)
			Dim bytes() As Byte
			Dim r As Integer = 0
			Dim c As Integer = 0
			Dim p As Integer = 0

			Dim file As New FileStream(path, FileMode.Open, FileAccess.Read)
            bytes = New Byte(Convert.ToInt32(file.Length - 2)) {}
			file.Read(bytes,0,bytes.Length)
			file.Close()

			Do While r < 9
				c = 0
				Do While c < 9 AndAlso p < bytes.Length
					Grid(r,c) = 0
					Dim n As Integer = CInt(bytes(p))
					p += 1
					If n > 48 Then
						Grid(r,c) = n-48
						WorkingGrid(r,c) = n-48
						c += 1
					ElseIf n = 48 Then
						Grid(r,c) = 0
						WorkingGrid(r,c) = 0
						c += 1
					End If
				Loop
				r += 1
			Loop
			_solved = False
		End Sub
		Public ReadOnly Property Grid() As Integer(,)
			Get
				If _grid Is Nothing Then
					_grid = New Integer(8, 8){}
				End If
				Return _grid
			End Get
		End Property
		Public ReadOnly Property WorkingGrid() As Integer(,)
			Get
				If _workingGrid Is Nothing Then
					_workingGrid = New Integer(8, 8){}
				End If
				Return _workingGrid
			End Get
		End Property
		Public ReadOnly Property IsSolved() As Boolean
			Get
				_solved = ColsCheck() AndAlso RowsCheck() AndAlso SubGridsCheck()
				Return _solved
			End Get
		End Property

		Private Function ColsCheck() As Boolean
			For c As Integer = 0 To 8
				If Not ColCheck(c) Then
					Return False
				End If
			Next c
			Return True
		End Function

		Private Function ColCheck(ByVal col As Integer) As Boolean
			Dim Values(8) As Integer
			For r As Integer = 0 To 8
				Dim n As Integer = _grid(r,col)
				If n = 0 Then
					n = _workingGrid(r,col)
				End If
				Values(r) = n
			Next r
			Return UniqueValueCheck(Values)
		End Function


		Private Function SubGridsCheck() As Boolean
			For r As Integer = 0 To 8 Step 3
				For c As Integer = 0 To 8 Step 3
					If Not SubGridCheck(r,c) Then
						Return False
					End If
				Next c
			Next r
			Return True
		End Function

		Private Function SubGridCheck(ByVal row As Integer, ByVal col As Integer) As Boolean
			Dim Values(8) As Integer
			Dim n As Integer = 0
			For r As Integer = row To row + 3 - 1
				For c As Integer = col To col + 3 - 1
					Dim i As Integer = _grid(r,c)
					If i = 0 Then
						i = _workingGrid(r,c)
					End If
					Values(n) = i
					n += 1
				Next c
			Next r
			Return UniqueValueCheck(Values)
		End Function

		Private Function UniqueValueCheck(ByVal Values() As Integer) As Boolean
			For n1 As Integer = 0 To Values.Length - 1
				For n2 As Integer = n1 + 1 To Values.Length - 1
					If Values(n1) = 0 Then
						Return False
					End If
					If Values(n1) = Values(n2) Then
						Return False
					End If
				Next n2
			Next n1
			Return True
		End Function

		Private Function RowsCheck() As Boolean
			For r As Integer = 0 To 8
				If Not RowCheck(r) Then
					Return False
				End If
			Next r
			Return True
		End Function

		Private Function RowCheck(ByVal row As Integer) As Boolean
			Dim Values(8) As Integer
			For c As Integer = 0 To 8
				Dim n As Integer = _grid(row,c)
				If n = 0 Then
					n = _workingGrid(row,c)
				End If
				Values(c) = n
			Next c
			Return UniqueValueCheck(Values)
		End Function
	End Class
End Namespace
