Imports System.Xml
Imports System.Web.UI
Imports System.Web.UI.WebControls
Imports System.Web.UI.HtmlControls
Imports Dart.LiveControls

Namespace LiveSudoku
	''' <summary>
	''' Class to hold Skin data to control a puzzle's GUI
	''' </summary>
	Public Class Skin
		Private doc As XmlDocument
		Private _id, _name, _description As String
		Public Sub New(ByVal xd As XmlDocument)
			Me.New(xd.OuterXml)
		End Sub
		Public Sub New(ByVal skinData As String)
			Try
				doc = New XmlDocument()
				doc.LoadXml(skinData)
				loadData()
			Catch
				Throw New ArgumentException("Invalid skin file.")
			End Try
		End Sub
		Private Sub loadData()
			Dim docEl As XmlElement = doc.DocumentElement
			_id = docEl.GetAttribute("id")
			_name = docEl.GetAttribute("name")
			_description = docEl.GetAttribute("description")
		End Sub

		Public ReadOnly Property ID() As String
			Get
				Return _id
			End Get
		End Property
		Public ReadOnly Property Name() As String
			Get
				Return _name
			End Get
		End Property
		Public ReadOnly Property Description() As String
			Get
				Return _description
			End Get
		End Property

		Public ReadOnly Property BackColor() As Color
			Get
				Dim hex As String = backGround.GetAttribute("color")
				Return StringToColor(hex)
			End Get
		End Property

		Private Function StringToColor(ByVal hex As String) As Color
			If hex <> "" Then
				hex = hex.Replace("#","")
				Dim hexr As String = hex.Substring(0,2)
				Dim hexg As String = hex.Substring(2,2)
				Dim hexb As String = hex.Substring(4,2)
				Return Color.FromArgb(Convert.ToInt32(hexr,16),Convert.ToInt32(hexg,16),Convert.ToInt32(hexb,16))
			Else
				Return Color.Transparent
			End If
		End Function

		Public ReadOnly Property BackImageUrl() As String
			Get
				Return backGround.GetAttribute("imageUrl")
			End Get
		End Property

		Public ReadOnly Property TileExtension() As String
			Get
				Return tiles.GetAttribute("ext")
			End Get
		End Property

		Public ReadOnly Property SolvedWidth() As Unit
			Get
				Return New Unit(solved.GetAttribute("width"))
			End Get
		End Property
		Public ReadOnly Property SolvedHeight() As Unit
			Get
				Return New Unit(solved.GetAttribute("height"))
			End Get
		End Property
		Public ReadOnly Property TileWidth() As Unit
			Get
				Return New Unit(tiles.GetAttribute("width"))
			End Get
		End Property
		Public ReadOnly Property TileHeight() As Unit
			Get
				Return New Unit(tiles.GetAttribute("height"))
			End Get
		End Property
		Public ReadOnly Property MenuWidth() As Unit
			Get
				Return New Unit(menu.GetAttribute("width"))
			End Get
		End Property
		Public ReadOnly Property MenuSpacing() As Integer
			Get
				Return Integer.Parse(menu.GetAttribute("spacing"))
			End Get
		End Property
		Public ReadOnly Property PopupButtonWidth() As Unit
			Get
				Return New Unit(popup.GetAttribute("buttonWidth"))
			End Get
		End Property
		Public ReadOnly Property PopupButtonHeight() As Unit
			Get
				Return New Unit(popup.GetAttribute("buttonHeight"))
			End Get
		End Property
		Public ReadOnly Property PopupBackColor() As Color
			Get
				Return StringToColor(popup.GetAttribute("bgcolor"))
			End Get
		End Property
		Public ReadOnly Property MenuLocation() As String
			Get
				Return menu.GetAttribute("location")
			End Get
		End Property

		Public ReadOnly Property MenuBackColor() As Color
			Get
				Return StringToColor(menu.GetAttribute("bgcolor"))
			End Get
		End Property
		Public ReadOnly Property MenuBackImageUrl() As String
			Get
				Return menu.GetAttribute("bgimageUrl")
			End Get
		End Property

		Public Sub SetControlStyle(ByVal ctl As WebControl, ByVal style As String)
			Dim el As XmlElement = CType(menu.SelectSingleNode(style), XmlElement)
			If el IsNot Nothing Then
				ctl.ControlStyle.Reset()
				ctl.MergeStyle(GenControl(el).ControlStyle)
			End If
		End Sub
		Private ReadOnly Property backGround() As XmlElement
			Get
				Return CType(doc.SelectSingleNode("skin/background"), XmlElement)
			End Get
		End Property
		Private ReadOnly Property tiles() As XmlElement
			Get
				Return CType(doc.SelectSingleNode("skin/tiles"), XmlElement)
			End Get
		End Property
		Private ReadOnly Property menu() As XmlElement
			Get
				Return CType(doc.SelectSingleNode("skin/menu"), XmlElement)
			End Get
		End Property
		Private ReadOnly Property popup() As XmlElement
			Get
				Return CType(doc.SelectSingleNode("skin/popup"), XmlElement)
			End Get
		End Property
		Private ReadOnly Property solved() As XmlElement
			Get
				Return CType(doc.SelectSingleNode("skin/solved"), XmlElement)
			End Get
		End Property
		Private Function GenControl(ByVal fontEl As XmlElement) As WebControl
			Dim l As New LiveLabel()
			l.Font.Name = fontEl.GetAttribute("fontName")
			l.Font.Size = New FontUnit(fontEl.GetAttribute("fontSize"))
			l.Font.Bold = Boolean.Parse(fontEl.GetAttribute("fontBold"))
			l.ForeColor = StringToColor(fontEl.GetAttribute("fontColor"))
			l.BackColor = StringToColor(fontEl.GetAttribute("bgcolor"))
			Return CType(l, WebControl)
		End Function
	End Class
End Namespace
